package no.ntnu.idatt1002.demo.data.Economics;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.util.ArrayList;

import static org.junit.jupiter.api.Assertions.*;

public class ItemRegisterTest {
    ItemRegister<Income> incomeRegister;
    ItemRegister<Expense> expenseRegister;

    @Nested
    @DisplayName("Test ItemRegister when using Income")
    class incomeRegisterTests {
        @BeforeEach
        public void createItemOverView() {
            incomeRegister = new ItemRegister();
        }

        @Test
        @DisplayName("addItem method throws exception when it should")
        void addItemThrows() {
            Income income = new Income("description", 59.9f, false, IncomeCategory.SALARY, "03.03.23");
            assertThrows(IllegalArgumentException.class, () -> {
                incomeRegister.addItem(income);
                incomeRegister.addItem(income);
            });
        }

        @Test
        @DisplayName("addItem method does not throw exception when it should not")
        void addItemDoesNotThrow() {
            Income income1 = new Income("description", 59.9f, false, IncomeCategory.GIFT, "03.03.23");
            Income income2 = new Income("anotherDescription", 6.5f, true, IncomeCategory.SALARY, "02.03.23");
            assertDoesNotThrow(() -> {
                incomeRegister.addItem(income1);
                incomeRegister.addItem(income2);
            });
        }

        @Test
        @DisplayName("getTotalSum method gives correct amount")
        void getTotalSumCorrectAmount() {
            Income income1 = new Income("description1", 59.9f, false, IncomeCategory.SALARY, "03.03.23");
            Income income2 = new Income("description2", 62.4f, true, IncomeCategory.GIFT, "01.02.21");
            Income income3 = new Income("description3", 9.81f, false, IncomeCategory.SALARY, "05.07.23");
            incomeRegister.addItem(income1);
            incomeRegister.addItem(income2);
            incomeRegister.addItem(income3);

            double totalIncome = 59.9f + 62.4f + 9.81f;
            assertEquals(Math.round(incomeRegister.getTotalSum()), Math.round(totalIncome));
        }

        @Test
        void getIncomeBasedOnCategoryGivesCorrectIncome() {
            Income income1 = new Income("description1", 59.9f, false, IncomeCategory.SALARY, "03.03.23");
            Income income2 = new Income("description2", 62.4f, true, IncomeCategory.GIFT, "01.02.21");
            Income income3 = new Income("description3", 9.81f, false, IncomeCategory.SALARY, "05.07.23");

            incomeRegister.addItem(income1);
            incomeRegister.addItem(income2);
            incomeRegister.addItem(income3);
            ArrayList<Income> incomeSalary = incomeRegister.getItemsBasedOnCategory(IncomeCategory.SALARY);
            assertTrue(incomeSalary.contains(income1) && incomeSalary.contains(income3));
        }
    }

    @Nested
    @DisplayName("Test ItemRegister when using Expense")
    class expenseRegisterTests {
        @BeforeEach
        public void createItemOverView() {
            expenseRegister = new ItemRegister();
        }

        @Test
        @DisplayName("addItem method throws exception when it should")
        void addItemThrows() {
            Expense expense = new Expense("description", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
            assertThrows(IllegalArgumentException.class, () -> {
                expenseRegister.addItem(expense);
                expenseRegister.addItem(expense);
            });
        }

        @Test
        @DisplayName("addItem method does not throw exception when it should not")
        void addItemDoesNotThrow() {
            Expense expense1 = new Expense("description", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
            Expense expense2 = new Expense("anotherDescription", 6.5f, true, ExpenseCategory.BOOKS, "02.03.23");
            assertDoesNotThrow(() -> {
                expenseRegister.addItem(expense1);
                expenseRegister.addItem(expense2);
            });
        }

        @Test
        @DisplayName("getTotalSum method gives correct amount")
        void getTotalSumCorrectAmount(){
            Expense expense1 = new Expense("description1", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
            Expense expense2 = new Expense("description2", 62.4f, true, ExpenseCategory.FOOD, "01.02.21");
            Expense expense3 = new Expense("description3", 9.81f, false, ExpenseCategory.CLOTHES, "05.07.23");

            expenseRegister.addItem(expense1);
            expenseRegister.addItem(expense2);
            expenseRegister.addItem(expense3);
            double totalIncome = 59.9f + 62.4f + 9.81f;
            assertEquals(Math.round(expenseRegister.getTotalSum()), Math.round(totalIncome));
        }

        @Test
        void getIncomeBasedOnCategoryGivesCorrectIncome() {
            Expense expense1 = new Expense("description1", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
            Expense expense2 = new Expense("description2", 62.4f, true, ExpenseCategory.FOOD, "01.02.21");
            Expense expense3 = new Expense("description3", 9.81f, false, ExpenseCategory.CLOTHES, "05.07.23");

            expenseRegister.addItem(expense1);
            expenseRegister.addItem(expense2);
            expenseRegister.addItem(expense3);
            ArrayList<Expense> expenseSalary = expenseRegister.getItemsBasedOnCategory(ExpenseCategory.CLOTHES);
            assertTrue(expenseSalary.contains(expense1) && expenseSalary.contains(expense3));
        }
    }
}
 